unit BZ2Main;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
  Dialogs, StdCtrls, Buttons, ExtCtrls, ComCtrls;

type
  TLogoAppForm = class(TForm)
    LogoAppMenu: TMainMenu;
    FileMenu: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveItem: TMenuItem;
    FileSendItem: TMenuItem;
    FileExitItem: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Help1: TMenuItem;
    AboutItem: TMenuItem;
    SpeedPanel: TPanel;
    OpenBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    StatusBar: TStatusBar;
    RichEdit1: TRichEdit;
    SendBtn: TSpeedButton;
    OpenCompressed1: TMenuItem;
    SaveCompressed1: TMenuItem;
    SaveCompressedAs1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure FileNew(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSave(Sender: TObject);
    procedure FileSaveAs(Sender: TObject);
    procedure FileSend(Sender: TObject);
    procedure About(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure OpenCompressed1Click(Sender: TObject);
    procedure SaveCompressed1Click(Sender: TObject);
    procedure SaveCompressedAs1Click(Sender: TObject);
  private
    FFileName: String;
    procedure LoadCompFromStream(Stream: TStream);
    procedure LoadUncompFromStream(Stream: TStream);
    procedure LoadCompFromFile(const AFileName: string);
    procedure SaveCompToStream(Stream: TStream);
    procedure SaveUncompToStream(Stream: TStream);
    procedure SaveCompToFile(const AFileName: string);
  public
    { Public declarations }
  end;

var
  LogoAppForm: TLogoAppForm;

implementation

uses SysUtils, Mapi, BZ2About, LogoStrs, BZip2;

{$R *.DFM}

procedure TLogoAppForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
end;

procedure TLogoAppForm.FileNew(Sender: TObject);
begin
  FFileName := LoadStr(sUntitled);
  RichEdit1.Lines.Clear;
  RichEdit1.Modified := False;
end;

procedure TLogoAppForm.FileOpen(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    RichEdit1.Lines.LoadFromFile(OpenDialog.FileName);
    FFileName := OpenDialog.FileName;
    RichEdit1.SetFocus;
    RichEdit1.Modified := False;
    RichEdit1.ReadOnly := ofReadOnly in OpenDialog.Options;
  end;
end;

procedure TLogoAppForm.FileSave(Sender: TObject);
begin
  if (FFileName = LoadStr(sUntitled)) or (FFileName = '') then
    FileSaveAs(Sender)
  else
  begin
    RichEdit1.Lines.SaveToFile(FFileName);
    RichEdit1.Modified := False;
  end;
end;

procedure TLogoAppForm.FileSaveAs(Sender: TObject);
begin
  with SaveDialog do
  begin
    FileName := FFileName;
    if Execute then
    begin
      if FileExists(FileName) then
        if MessageDlg(Format(LoadStr(sOverwrite), [FileName]),
          mtConfirmation, mbYesNoCancel, 0) <> idYes then Exit;
      RichEdit1.Lines.SaveToFile(FileName);
      FFileName := FileName;
      RichEdit1.Modified := False;
    end;
  end;
end;

procedure TLogoAppForm.FileSend(Sender: TObject);
var
  MapiMessage: TMapiMessage;
  MError: Cardinal;
begin
  with MapiMessage do
  begin
    ulReserved := 0;
    lpszSubject := nil;
    lpszNoteText := PChar(RichEdit1.Lines.Text);
    lpszMessageType := nil; 
    lpszDateReceived := nil; 
    lpszConversationID := nil; 
    flFlags := 0;
    lpOriginator := nil; 
    nRecipCount := 0;
    lpRecips := nil; 
    nFileCount := 0;
    lpFiles := nil; 
  end;
  
  MError := MapiSendMail(0, 0, MapiMessage, 
    MAPI_DIALOG or MAPI_LOGON_UI or MAPI_NEW_SESSION, 0);
  if MError <> 0 then MessageDlg(LoadStr(sSendError), mtError, [mbOK], 0);
end;

procedure TLogoAppForm.FileExit(Sender: TObject);
begin
  Close;
end;

procedure TLogoAppForm.About(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TLogoAppForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;

procedure TLogoAppForm.OpenCompressed1Click(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    FFileName := OpenDialog.FileName;
    LoadCompFromFile(FFileName);
    RichEdit1.SetFocus;
    RichEdit1.Modified := False;
    RichEdit1.ReadOnly := ofReadOnly in OpenDialog.Options;
  end;
end;

procedure TLogoAppForm.SaveCompressed1Click(Sender: TObject);
begin
  if (FFileName = LoadStr(sUntitled)) or (FFileName = '') then
    SaveCompressedAs1Click(Sender)
  else
  begin
    SaveCompToFile(FFileName);
    RichEdit1.Modified := False;
  end;
end;

procedure TLogoAppForm.SaveCompressedAs1Click(Sender: TObject);
begin
  with SaveDialog do
  begin
    FileName := FFileName;
    if Execute then
    begin
      if FileExists(FileName) then
        if MessageDlg(Format(LoadStr(sOverwrite), [FileName]),
          mtConfirmation, mbYesNoCancel, 0) <> idYes then Exit;
      SaveCompToFile(FileName);
      FFileName := FileName;
      RichEdit1.Modified := False;
    end;
  end;
end;

procedure TLogoAppForm.LoadUncompFromStream(Stream: TStream);
begin
    RichEdit1.Lines.LoadFromStream(Stream);
end;

procedure TLogoAppForm.LoadCompFromFile(const AFileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadCompFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TLogoAppForm.LoadCompFromStream(Stream: TStream);
const
  BufferSize = 65536;
var
  DecompressionStream: TStream;
  Dest: TStream;
  Count: Integer;
  Buffer: array[0..BufferSize - 1] of Byte;
begin
  Dest := TMemoryStream.Create;
  try
    DecompressionStream := TBZDecompressionStream.Create(Stream);
    try
      while True do
      begin
        Count := DecompressionStream.Read(Buffer, BufferSize);
        if Count <> 0 then Dest.WriteBuffer(Buffer, Count) else Break;
      end;
    finally
      DecompressionStream.Free;
    end;
    Dest.Position := 0;
    LoadUncompFromStream(Dest);
  finally
    Dest.Free;
  end;
end;

procedure TLogoAppForm.SaveCompToStream(Stream: TStream);
var
  Source: TStream;
  CompressionStream: TStream;
begin
  Source := TMemoryStream.Create;
  try
    SaveUncompToStream(Source);
    CompressionStream := TBZCompressionStream.Create(bs9, Stream);
    try
      CompressionStream.CopyFrom(Source, 0);
    finally
      CompressionStream.Free;
    end;
  finally
    Source.Free;
  end;
end;

procedure TLogoAppForm.SaveUncompToStream(Stream: TStream);
begin
    RichEdit1.Lines.SaveToStream(Stream);
end;

procedure TLogoAppForm.SaveCompToFile(const AFileName: string);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(AFileName, fmCreate);
  try
    SaveCompToStream(Stream);
  finally
    Stream.Free;
  end;
end;

end.
 
